/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.parameterized;

import org.apache.commons.lang.NullArgumentException;

/**
 * @author jbunting
 * @TODO Document Me!
 * Created by IntelliJ IDEA.
 * Date: Mar 3, 2008
 * Time: 10:52:28 AM
 */
public class AppendParameterSet extends AbstractParameterSet {
  protected String defaultName() {
    return null;  //To change body of implemented methods use File | Settings | File Templates.
  }

  private ParameterSet[] subSets;

  public AppendParameterSet(ParameterSet[] subSets) {
    this(null, null, subSets);
  }
  
  public AppendParameterSet(String name, String[] labelArray, ParameterSet[] subSets) {
    super(name, labelArray);
    this.subSets = subSets;
  }

  protected String defaultLabel(int iteration) {
    SubIterationHolder subIteration = getSubIteration(iteration);
    return String.format("%s[%s]", subIteration.getSubDefinition().getName(), subIteration.getLabel());
  }

  public int getNumIterations() {
    int numIterations = 0;
    for(ParameterSet subSet : subSets) {
      numIterations += subSet.getNumIterations();
    }
    return numIterations;
  }

  public int getIterationSize() {
    return subSets[0].getIterationSize();
  }

  public Object[] getIteration(int iteration) {
    return getSubIteration(iteration).getSubIteration();
  }

  public void validate() {
    if(subSets.length < 1) {
      throw new IllegalArgumentException("At least one sub definition must be specified!");
    }
    for(ParameterSet definition : subSets) {
      if(definition == null) {
        throw new NullArgumentException("Sub definitions cannot be null!");
      }
    }
    int iterationSize = subSets[0].getIterationSize();
    for(int i = 1; i < subSets.length; i++) {
      if(subSets[i].getIterationSize() != iterationSize) {
        throw new IllegalArgumentException("All sub definitions must have the same iteration size!");
      }
    }
  }

  protected SubIterationHolder getSubIteration(int iteration) {
    for(ParameterSet subSet : subSets) {
      if(subSet.getNumIterations() > iteration) {
        return new SubIterationHolder(subSet, iteration);
      } else {
        iteration -= subSet.getNumIterations();
      }
    }
    throw new IllegalArgumentException("Iteration value " + iteration + " is invalid (too high).");
  }

  private class SubIterationHolder {
    private ParameterSet subSet;
    private int iteration;

    private SubIterationHolder(ParameterSet subSet, int iteration) {
      this.subSet = subSet;
      this.iteration = iteration;
    }

    public ParameterSet getSubDefinition() {
      return subSet;
    }

    public int getIteration() {
      return iteration;
    }

    public Object[] getSubIteration() {
      return subSet.getIteration(iteration);
    }

    public String getLabel() {
      return subSet.getIterationLabel(iteration);
    }
  }
}
