/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.parameterized;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;

/**
 * @author jbunting
 * @TODO Document Me!
 * Created by IntelliJ IDEA.
 * Date: Mar 3, 2008
 * Time: 8:26:43 AM
 */
public interface ParameterSet {

  String getName();

  int getNumIterations();

  int getIterationSize();

  Object[] getIteration(int iteration);

  String getIterationLabel(int iteration);

  void validate();

  void setLabels(String[] labels);

  void setName(String name);

  /**
 * Indicates that a field represents a parameter set.  The field must be public, static, and final.
   * Date: Feb 19, 2008
   * Time: 3:11:49 PM
   *
   * @author jbunting
   * @see org.xj4.XJ4Runner
   */

  @Documented
  @Retention(RetentionPolicy.RUNTIME)
  @Target(ElementType.FIELD)
  @Inherited
  public static @interface As {

    /**
     * The labels that will be used in naming the iterations of the test.  Any iterations that don't have an apporpriate
     * label will be named with their iteration index.
     * @return the labels
     */
    public abstract String[] labels() default {};

    /**
     * If this is set to true, then the engine will assume that the first column in the specified object array is the
     * labels and will not treat them as parameters.  Note that this field will be ignored if this annotation is
     * placed on a field that does not represent a two dimensional array.
     * @return true if the first column should be treated as the labels for the parameter set
     */
    public boolean labelsIncluded() default false;
  }

  @Documented
  @Retention(RetentionPolicy.RUNTIME)
  @Target(ElementType.FIELD)
  @Inherited
  public static @interface Labels {
    public abstract String[] value() default {};
  }
}
