/**
 * Copyright 2008 Peach Jean Solutions
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.xj4.parameterized;

import org.apache.commons.lang.ArrayUtils;

/**
 * @author jbunting
 * @TODO Document Me!
 * Created by IntelliJ IDEA.
 * Date: Mar 3, 2008
 * Time: 11:20:38 AM
 */
public class ParameterSetUtils {

  public static ParameterSet create(Object[][] values) {
    return create(values, false);
  }

  public static ParameterSet create(Object[][] values, boolean labelsIncluded) {
    String[] labels = null;
    if(labelsIncluded) {
      labels = new String[values.length];
      Object[][] newValues = new Object[values.length][];
      for(int i = 0; i < values.length; i++) {
        labels[i] = values[i].length > 0 ? values[i][0].toString() : Integer.toString(i);
        newValues[i] = ArrayUtils.subarray(values[i], 1, values[i].length);
      }
      values = newValues;
    }
    return validate(new MultiValueParameterSet(labels, values));
  }

  public static ParameterSet create(Object[] values) {
    return validate(new SingleValueParameterSet(values));
  }

  public static ParameterSet create(Object[] values, String[] labels) {
    return validate(new SingleValueParameterSet(values, labels));
  }

  public static ParameterSet create(Object value) {
    return create(new Object[] { value });
  }

  public static ParameterSet create(Object value, String label) {
    return create(new Object[] { value }, new String[] { label });
  }

  public static ParameterSet append(ParameterSet... subSets) {
    return validate(new AppendParameterSet(subSets));
  }

  public static ParameterSet simpleMerge(ParameterSet... subSets) {
    return validate(new SimpleMergeParameterSet(subSets));
  }

  public static ParameterSet simpleMerge(int labelsFrom, ParameterSet... subSets) {
    ParameterSet labelSource = subSets[labelsFrom];
    String[] labelArray = new String[labelSource.getNumIterations()];
    for(int i = 0; i < labelArray.length; i++) {
      labelArray[i] = labelSource.getIterationLabel(i);
    }
    return validate(new SimpleMergeParameterSet(labelArray, subSets));
  }

  public static ParameterSet index(ParameterSet subSet) {
    return validate(new IndexParameterSet(subSet));
  }

  public static ParameterSet indexAndMerge(ParameterSet subSet) {
    return simpleMerge(1, index(subSet), subSet);
  }

  public static ParameterSet cartesian(ParameterSet... subSets) {
    return validate(new CartesianParameterSet(subSets));
  }

  protected static ParameterSet validate(ParameterSet definition) {
    definition.validate();
    return definition;
  }
}
